@@ -219,9 +219,12 @@ public class GObject {
      * org.gnu.glib.Value} on this object.  If there is a GTK property with 
      * this name, the GTK property will be set.  Otherwise, a Java-side
      * property will be set.
+     * <p>
+     * <b>NOTE</b>: Prefer using one of the convenience methods over using
+     * this method directly.
      *
      * @param name The name of the property to set.
-     * @param value The value to set in the property.
+     * @param val The value to set in the property.
      */
     public void setProperty( String name, Value val ) {
         if ( hasGtkProperty( name ) ) {
@@ -316,24 +319,33 @@ public class GObject {
         val.setPixbuf( value );
         setProperty( name, val );
     }
+
     /**
      * Convenience method for setting <tt>Object</tt> properties.
      *
      * @param name The name of the property to set.
      * @param value The value to set in the property.
-     * @see #setProperty( String, Value )
      */
     public void setJavaObjectProperty( String name, Object value ) {
-        Value val = new Value( Type.OBJECT() );
+        Value val = null;
+        if ( hasGtkProperty( name ) ) {
+            Type type = new Type( getGTypeOfProperty( getHandle(), name ) );
+            val = new Value( type );
+        } else {
+            val = new Value( Type.OBJECT() );
+        }
         val.setJavaObject( value );
         setProperty( name, val );
     }
 
     /**
      * Get the {@link org.gnu.glib.Value} of the given property
-     * (<tt>name<tt>).  If there is a GTK property with this name, the
+     * (<tt>name</tt>).  If there is a GTK property with this name, the
      * GTK property will be retrived.  Otherwise, a Java-side property will
      * be returned.
+     * <p>
+     * <b>NOTE</b>: Prefer using one of the convenience methods over using
+     * this method directly.
      *
      * @param name The name of the property to retrieve.
      * @return The value of the given property.
@@ -452,16 +464,32 @@ public class GObject {
      */
     public Object getJavaObjectProperty( String name ) {
         Value val = getProperty( name );
-        if ( val != null ) {
-            return val.getJavaObject();
-        } else {
+        if ( val == null ) {
             return null;
         }
+
+        Object obj = val.getJavaObject();
+        if ( obj == null ) {
+            return null;
+        }
+
+        if ( obj instanceof Handle ) {
+            // I would like to use:
+            //GObject gobj = GObject.getGObjectFromHandle( (Handle)obj );
+            //
+            // But this seems to crash the JVM in getData, so for now we 
+            // always instantiate a new object.
+            int type = getGTypeOfProperty( getHandle(), name );
+            return instantiateJGObjectFromGType( type, (Handle)obj );
+
+        } else {
+            return obj;
+        }
     }
 
     /**
-     * Does this object supported the given named property?  First the
-     * GTK properties are check, then, if no property with this name is
+     * Determines if this object supports the given named property.  First the
+     * GTK properties are checked, then, if no property with this name is
      * found, the Java-side properties are checked.
      *
      * @param name The property to verify.
@@ -677,6 +705,22 @@ public class GObject {
     native static final private void g_object_thaw_notify(Handle object);
     native static final private void connectNotifySignal(Handle objhandle, GObject object, String callback);
     native static final private void disconnectNotifySignal(Handle object);
+    native static final private int getGTypeOfProperty(Handle object, String prop);
+    // Perhaps this could/should be public in the future??
+    native static final protected Object instantiateJGObjectFromGType(int type, Handle handle);
+
+    /**
+     * Return the int to which the native {@link org.gnu.glib.Handle}
+     * holds a pointer.  The native Handle is assumed to hold a C
+     * pointer to an int ( gint* ).  This method dereferences that
+     * pointer and returns the value to which it points as an integer.
+     *
+     * @param inthandle The native Handle holding a pointer to an int.
+     * @return The int that is pointed to by the native Handle.
+     */
+    native static final protected int getIntFromHandle(Handle inthandle);
+
+    //native static final public void setTestProperty(Handle object, String prop, Handle obj);
 
     // ------------------------------------------------------------------------
     // static init code
