@@ -37,53 +37,6 @@ typedef struct {
   CallbackData *cbdata;
 } JGClosure;
 
-
-static char *
-javatype_from_gtktype(GType argtype)
-{
-  char *javatype = g_malloc(50);
-  const char *tmp;
-
-  tmp = g_type_name(argtype);
-    
-  if (strncmp(tmp, "Gtk", 3) == 0) {
-    if (strlen(tmp) > 3)
-      tmp = &tmp[3];
-    sprintf(javatype, "org/gnu/gtk/%s", tmp);
-  }
-  else if (strncmp(tmp, "Gdk", 3) == 0) {
-    if (strlen(tmp) > 3)
-      tmp = &tmp[3];
-    sprintf(javatype, "org/gnu/gdk/%s", tmp);
-  }
-  else if (strncmp(tmp, "Gno", 3) == 0) {
-    if (strlen(tmp) > 5)
-      tmp = &tmp[5];
-    sprintf(javatype, "org/gnu/gnome/%s", tmp);
-  }
-  else if (strncmp(tmp, "Pan", 3) == 0) {
-    if (strlen(tmp) > 5)
-      tmp = &tmp[5];
-    sprintf(javatype, "org/gnu/pango/%s", tmp);
-  }
-  else if (strncmp(tmp, "Atk", 3) == 0) {
-    if (strlen(tmp) > 3)
-      tmp = &tmp[3];
-    sprintf(javatype, "org/gnu/atk/%s", tmp);
-  }
-  else if (strncmp(tmp, "Html", 4) == 0) {
-    sprintf(javatype, "org/gnu/gtkhtml/%s", tmp);
-  }
-  else if (strncmp(tmp, "Dom", 3) == 0) {
-    sprintf(javatype, "org/gnu/gtkhtml/dom/%s", tmp);
-  }
-  else
-    sprintf(javatype, "%s", tmp);
-
-  return javatype;
-}
-
-
 /* This method actually recieves all callbacks in the system.  It 
  * recieves a CallbackData struct as its' data.  It then determines
  * what Java method to call and invokes it passing the correct parameters */
@@ -174,19 +127,8 @@ jg_signal_cb(GClosure *closure,
       jargs[i-1].l = peer;
       break;
     default:
-      jtype = javatype_from_gtktype(G_VALUE_TYPE(&param_values[i]));
-      clazz = (* cbdata->env)->FindClass(cbdata->env, jtype);
-      g_free((gpointer)jtype);
-      if (clazz == NULL) {
-		return;
-      }
-    
-      localObj = (* cbdata->env)->AllocObject(cbdata->env, clazz);
       peer = getHandleFromPointer(cbdata->env, g_value_get_object(&param_values[i]));
-      peerfid = (* cbdata->env)->GetFieldID(cbdata->env, clazz, 
-					    "handle", "Lorg/gnu/glib/Handle");
-      (* cbdata->env)->SetObjectField(cbdata->env, localObj, peerfid, peer);
-      jargs[i-1].l = localObj;
+      jargs[i-1].l = peer;
       break;
     }
   }
@@ -314,13 +256,13 @@ jg_arg_signature(GString *str, GType argtype)
     as = "org/gnu/glib/Handle";
     break;
   case G_TYPE_POINTER:
-    ac = 'I';
+    as = "org/gnu/glib/Handle";
     break;
   case G_TYPE_OBJECT:
     as = "org/gnu/glib/Handle";
     break;
   default:
-    ac = 'I';
+    as = "org/gnu/glib/Handle";
     break;
   case G_TYPE_INVALID:
     g_error("Type %d - should not be reached", (gint)argtype);
@@ -696,15 +638,15 @@ Java_org_gnu_glib_GObject_removeEventHandler(JNIEnv *env, jobject obj, jint sigi
 JNIEXPORT jobject JNICALL Java_org_gnu_glib_GObject_getData
   (JNIEnv *env, jobject obj, jobject handle, jstring key)
 {
-	gchar *k = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
-        GObject *handle_g = (GObject *)getPointerFromHandle(env, handle);
-	JGRef *ref = (JGRef*) g_object_get_data(handle_g, k);
-	(*env)->ReleaseStringUTFChars(env, key, k);
-	if (ref == NULL){
-		return NULL;
-	}else{
-		return ref->object;
-	}
+    gchar *k = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
+    GObject *handle_g = (GObject *)getPointerFromHandle(env, handle);
+    JGRef *ref = (JGRef*) g_object_get_data(handle_g, k);
+    (*env)->ReleaseStringUTFChars(env, key, k);
+    if (ref == NULL) {
+        return NULL;
+    } else {
+        return ref->object;
+    }
 }
 
 static void jg_gobject_destroy(gpointer data)
@@ -913,6 +855,113 @@ JNIEXPORT void JNICALL Java_org_gnu_glib_GObject_disconnectNotifySignal
     }
 }
 
+JNIEXPORT jint JNICALL Java_org_gnu_glib_GObject_getGTypeOfProperty(JNIEnv* env, jclass cls, jobject object, jstring prop) 
+{
+    GObject * object_g = (GObject *)getPointerFromHandle(env, object);
+    const gchar * prop_g = 
+        (const gchar *)(*env)->GetStringUTFChars(env, prop, 0);
+
+    GParamSpec *pspec = 
+        g_object_class_find_property(G_OBJECT_GET_CLASS(object_g), prop_g);
+    if ( !pspec ) {
+        return -1;
+    }
+
+    return pspec->value_type;
+}
+
+JNIEXPORT jobject JNICALL Java_org_gnu_glib_GObject_instantiateJGObjectFromGType(JNIEnv* env, jclass cls, int type, jobject handle) 
+{
+    jthrowable exc;
+    const gchar* jtype = javatype_from_gtktype((GType)type);
+    jclass clazz = (* env)->FindClass(env, jtype);
+    exc = (*env)->ExceptionOccurred(env);
+    if (exc) {
+        g_warning( "Can't find class: %s\n", jtype );
+    	(*env)->ExceptionDescribe(env);
+    	(*env)->ExceptionClear(env);
+        return NULL;
+    }
+    
+    if (clazz == NULL) {
+        clazz = (* env)->FindClass(env, "org/gnu/glib/GObject");
+        if (clazz == NULL) {
+            // Should never happen!
+            return NULL;
+        }
+    }
+    g_free((gpointer)jtype);
+    
+    jobject localObj = (* env)->AllocObject(env, clazz);
+    jmethodID handlemid = 
+        (* env)->GetMethodID(env, clazz, 
+                             "setHandle", "(Lorg/gnu/glib/Handle;)V");
+    exc = (*env)->ExceptionOccurred(env);
+    if (exc) {
+        g_warning( "No such method: setHandle(Lorg/gnu/glib/Handle;)V\n" );
+    	(*env)->ExceptionDescribe(env);
+    	(*env)->ExceptionClear(env);
+        return NULL;
+    }
+
+    (* env)->CallVoidMethod(env, localObj, handlemid, handle);
+    return localObj;
+}
+
+JNIEXPORT jint JNICALL Java_org_gnu_glib_GObject_getIntFromHandle(JNIEnv* env, jclass cls, jobject inthandle) 
+{
+    void *inthandle_g = getPointerFromHandle(env, inthandle);
+    return (jint) *(gint*)inthandle;
+}
+
+
+/*
+ * My test property setting method.  :-)
+ */
+JNIEXPORT void JNICALL Java_org_gnu_glib_GObject_setTestProperty(JNIEnv* env, jclass cls, jobject object, jstring prop, jobject val) 
+{
+    printf( "C-setTestProperty - start\n" );
+    GObject * object_g = (GObject *)getPointerFromHandle(env, object);
+    const gchar * prop_g = 
+        (const gchar *)(*env)->GetStringUTFChars(env, prop, 0);
+    gpointer val_g = (gpointer)getPointerFromHandle(env, val);
+    printf( "C-setTestProperty - obj_g=%d,prop_g=%s,val_g=%d\n", object_g, prop_g, val_g );
+
+    printf( "C-setTestProperty - g_object_class_find_property(obj_g,prop_g)\n" );
+    GParamSpec *pspec = 
+        g_object_class_find_property(G_OBJECT_GET_CLASS(object_g), prop_g);
+    if ( !pspec ) {
+        return;
+    }
+    printf( "C-setTestProperty - got pspec\n" );
+
+    printf( "C-setTestProperty - gval = g_malloc GValue\n" );
+    GValue *gval = (GValue *) g_malloc(sizeof(GValue));
+    gval->g_type = 0;
+    printf( "C-setTestProperty - g_value_init( gval=%d, pspec->value_type=%d )\n", gval, pspec->value_type );
+    gval = g_value_init( gval, pspec->value_type );
+    printf( "C-setTestProperty - inited\n" );
+
+    printf( "C-setTestProperty - g_value_set_ for gval=%d\n", gval );
+    if ( G_VALUE_HOLDS_BOXED( gval ) ) {
+        printf( "C-setTestProperty - HOLDS_BOXED.\n" );
+        g_value_set_boxed( gval, val_g );
+    } else if ( G_VALUE_HOLDS_OBJECT( gval ) ) {
+        printf( "C-setTestProperty - HOLDS_OBJECT.\n" );
+        g_value_set_object( gval, val_g );
+    } else if ( G_VALUE_HOLDS_POINTER( gval ) ) {
+        printf( "C-setTestProperty - HOLDS_POINTER.\n" );
+        g_value_set_pointer( gval, val_g );
+    } else {
+        printf( "C-setTestProperty - not anything!.\n" );
+        return;
+    }
+    printf( "C-setTestProperty - g_value_set completed.\n" );
+    printf( "C-setTestProperty - g_object_set_property(obj_g=%d, prop_g=%s, gval=%d).\n", object_g, prop_g, gval );
+    g_object_set_property(object_g, prop_g, gval );
+    printf( "C-setTestProperty - end\n" );
+}
+
 #ifdef __cplusplus
 }
 
